#include "HelloWorldScene.h"
#include "SimpleAudioEngine.h"

Scene* HelloWorld::createScene()
{
    // 'scene' is an autorelease object
    auto scene = Scene::create();
    
    // 'layer' is an autorelease object
    auto layer = HelloWorld::create();

    // add layer as a child to scene
    scene->addChild(layer);

    // return the scene
    return scene;
}

// on "init" you need to initialize your instance
bool HelloWorld::init()
{
    if ( !Layer::init() )
    {
        return false;
    }
    
    auto visibleSize = Director::getInstance()->getVisibleSize();
    Vec2 origin = Director::getInstance()->getVisibleOrigin();
    
    root = Node::create();
    addChild(root);

    FileUtils::getInstance()->addSearchPath("res");
    sprite = Sprite::create("image.png");
    sprite->setPosition(Vec2(visibleSize.width/2 + origin.x, visibleSize.height/2 + origin.y));
    sprite->setColor(Color3B::RED);
    root->addChild(sprite);
    
    auto listener = EventListenerTouchOneByOne::create();
    listener->setSwallowTouches(true);
    
    std::string str = "";
    for(int j = sprite->getContentSize().height; j >= 0; j--){
        for(int i = 0; i < sprite->getContentSize().width; i++){
            bool h = HitDetectHelper::hitTest(sprite, Vec2(i, j));
            str += h ? "#" : " ";
        }
        str += "\n";
    }
    
    CCLOG("%s", str.c_str());
    
    listener->onTouchBegan = [&](Touch* touch, Event* event){
        Point locationInNode = sprite->convertToNodeSpace(touch->getLocation());
        Point roundedLocation = Point(round(locationInNode.x), round(locationInNode.y));
        bool h = HitDetectHelper::hitTest(sprite, roundedLocation);
        CCLOG("onTouchBegan(%f, %f): %s", roundedLocation.x, roundedLocation.y, h ? "true" : "false");
        if(h){
            sprite->setColor(Color3B::GREEN);
        }
        else{
            sprite->setColor(Color3B::RED);
        }
        
        std::string str = "";
        for(int j = sprite->getContentSize().height; j >= 0; j--){
            for(int i = 0; i < sprite->getContentSize().width; i++){
                bool h = HitDetectHelper::hitTest(sprite, Vec2(i, j));
                str += h ? "#" : " ";
            }
            str += "\n";
        }
        
        CCLOG("%s", str.c_str());
        
        
        return true;
    };
    
    listener->onTouchMoved = [&](Touch* touch, Event* event){
        Point locationInNode = sprite->convertToNodeSpace(touch->getLocation());
        Point roundedLocation = Point(round(locationInNode.x), round(locationInNode.y));
        bool h = HitDetectHelper::hitTest(sprite, roundedLocation);
        CCLOG("onTouchMoved(%f, %f): %s", roundedLocation.x, roundedLocation.y, h ? "true" : "false");
        if(h){
            sprite->setColor(Color3B::GREEN);
        }
        else{
            sprite->setColor(Color3B::RED);
        }
    };
    
    listener->onTouchEnded = [&](Touch* touch, Event* event){
        sprite->setColor(Color3B::RED);
    };
    
    listener->onTouchCancelled = [&](Touch* touch, Event* event){
        sprite->setColor(Color3B::RED);
    };
    
    _eventDispatcher->addEventListenerWithSceneGraphPriority(listener, this);
    
    return true;
}
